/*
Rapid Doji EA 03

This MQL5 Expert Advisor uses a simple strategy on a daily chart.

For step-by-step video tutorials (with transcripts) on how this EA
was built, go to the link below.
*/
#property copyright		"Copyright 2020, Anthony J. Garot"
#property link			"https://www.garot.com/trading/"
#property version	 	"1.01"
#property description	""
#property description	"Rapid Doji EA 03"
#property description	""
#property description	"Entry: Two pending orders around each Daily Doji bar."
#property description	"Risk adjustment: Fixed trailing SL."
#property strict

#include <Trade\Trade.mqh>

#define NUM_RATES 3

//--- globals
CTrade			g_trade;					// trade object
double			g_pointSize = 0.;			// initialize to ludicrous value
// handles
int				g_handleAtr = 0;
// arrays
MqlRates		g_rates[];
double			g_atr[];

//--- input parameters
input double	SL_ATR_FACTOR = 1.75;
input int		InpAtrPeriod = 14;				// ATR Period

//--- constants
const long		MAGIC_NUMBER = 5678900;
const double	TRADE_VOLUME = 0.10;			// For strategy development, fixed lot size of 0.10 is good
const int		CHART_ID = 0;
const int		SUB_WINDOW = 0;

int OnInit(void)
{
	// netting mode check
	// 2020.02.12 - Because I never have two trades on the same symbol open simultaneously,
	// this check is unnecessary.
	if(AccountInfoInteger(ACCOUNT_MARGIN_MODE) != ACCOUNT_MARGIN_MODE_RETAIL_NETTING)
	{
		Print("This is a Hedging Account! NOT OK in the USA.");
		return INIT_FAILED;
	}

	g_trade.SetExpertMagicNumber(MAGIC_NUMBER);

	g_pointSize = SymbolInfoDouble(_Symbol, SYMBOL_POINT);
	PrintFormat("g_pointSize is [%.5f]", g_pointSize);

	//--- ATR
	g_handleAtr=iATR(	_Symbol,		// symbol
						_Period,		// period
						InpAtrPeriod );	// atr_period
	if ( g_handleAtr == INVALID_HANDLE )
	{
		PrintFormat("%s(): Error creating iATR indicator handle for [%s] [%d]", _Symbol, _Period );
		return INIT_FAILED;
	}

	ArrayResize(g_rates, NUM_RATES);
	ArraySetAsSeries(g_rates, true);
	ArrayResize(g_atr, NUM_RATES);
	ArraySetAsSeries(g_atr, true);

	//--- ok
	Print("INIT_SUCCEEDED for EA!");
	return(INIT_SUCCEEDED);
}

// Remember that this runs every tick -- unless the tester is running on "Open prices only"
void OnTick(void)
{
	// Bar level functionality
	if ( IsNewBar(_Symbol, _Period) )
	{
		// Need data or we've got nothing
		if ( ! RefreshData() )
		{
			Print("Couldn't refresh data");
			return;
		}

		// See if a position is already open
		if ( SelectPosition(_Symbol, MAGIC_NUMBER) )
		{
			// Position is open, so adjust risk as necessary
			// FixedTrailingStop();
		}
		else
		{
			// No position open, so look for signal
			CheckForOpenSignal();
		}

		// Draw every Doji whether we place a trade or not
		if ( IsDoji(g_rates[1]) )
		{
			DrawArrow(g_rates[1].time, g_rates[1].low - .333 * g_atr[1], ARROW_UP, Yellow, "Doji");
		}
	}
}

// OPEN LOGIC
// Returns true if a signal was found (not if position opened).
bool CheckForOpenSignal()
{
	// These are passed and filled in by the signal functions
	double			price = 0.;
	double			sl = 0.;			// initialize to no SL
	double			tp = 0.;			// initialize to no TP

	// BUY STOP
	if ( IsPendingBuySignal(price,sl,tp) )
	{
		//--- additional checking
		if ( Bars(_Symbol,_Period)>100 )
		{
			bool opened = OrderStop(	g_trade,
										_Symbol,
										ORDER_TYPE_BUY_STOP,
										TRADE_VOLUME,
										price,
										sl,
										tp );

			// Draw some lines
			if ( opened )
			{
				DrawLine ( sl, Brown, STYLE_DOT );
				DrawLine ( price, LightBlue, STYLE_DOT );
			}
		}
	}

	// SELL STOP
	if ( IsPendingSellSignal(price,sl,tp) )
	{
		//--- additional checking
		if ( Bars(_Symbol,_Period)>100 )
		{
			bool opened = OrderStop(	g_trade,
										_Symbol,
										ORDER_TYPE_SELL_STOP,
										TRADE_VOLUME,
										price,
										sl,
										tp );

			// Draw some lines
			if ( opened )
			{
				DrawLine ( sl, Brown, STYLE_DOT );
				DrawLine ( price, LightBlue, STYLE_DOT );
			}
		}
	}

	return false;
}

const uchar ARROW_UP = 233;
const uchar ARROW_DOWN = 234;
void DrawArrow(const datetime time, double price, const uchar code, const color clr, string comment)
{

	static int s_id = 0;		// to make unique string
	const string name = StringFormat("Arrow_%2d %s", s_id++, comment);

	if (!ObjectCreate( CHART_ID, name, OBJ_ARROW, SUB_WINDOW, time, price ))
	{
		PrintFormat("%s(): Failed to create OBJ_TEXT. Error code: [%d]", __FUNCTION__, GetLastError() );
		return;
	}

	ObjectSetInteger(CHART_ID, name, OBJPROP_COLOR,clr);
	ObjectSetInteger(CHART_ID, name, OBJPROP_ARROWCODE, code);
	ObjectSetInteger(CHART_ID, name, OBJPROP_WIDTH, 3);
}

void DrawLine( const double price, const color clr = Plum, const ENUM_LINE_STYLE lineStyle = STYLE_SOLID )
{
	static int s_id = 0;			// to make unique string
	const string name = StringFormat("Line_%2d Price [%.5f]", s_id++, price);

	if (!ObjectCreate( CHART_ID, name, OBJ_TREND, SUB_WINDOW, g_rates[2].time, price, g_rates[0].time, price))
	{
		PrintFormat("%s(): Failed to create OBJ_TREND. Error code: [%d]", __FUNCTION__, GetLastError() );
		return;
	}

	ObjectSetInteger(CHART_ID,name,OBJPROP_COLOR,clr);
	ObjectSetInteger(CHART_ID,name,OBJPROP_STYLE,lineStyle);
}



// Body <= 3% of Bar
// Does not consider position of body, e.g. gravestone or dragonfly
bool IsDoji(MqlRates& rate)
{
	const double DOJI_PCT = 0.03;		// Assume 3% or less body size for Doji

	const double sizeBody = MathAbs( rate.open - rate.close );
	const double sizeBar = ( rate.high - rate.low );
	return ( sizeBody <= DOJI_PCT * sizeBar );
}

// Detects when a "new bar" occurs, which is the same as when the previous bar has completed.
bool IsNewBar(const string symbol, const ENUM_TIMEFRAMES period)
{
	bool isNewBar = false;
	static datetime priorBarOpenTime = NULL;

	// New Bar event handler -> per https://www.mql5.com/en/articles/159
	// SERIES_LASTBAR_DATE == Open time of the last bar of the symbol-period
	const datetime currentBarOpenTime = (datetime) SeriesInfoInteger(symbol,period,SERIES_LASTBAR_DATE);

	if( priorBarOpenTime != currentBarOpenTime )
	{
		// Don't want new bar just because EA started
		if ( priorBarOpenTime == NULL )
		{
			isNewBar = false;
		}
		else
		{
			isNewBar = true;
		}
		// isNewBar = ( priorBarOpenTime == NULL )?false:true;	// priorBarOpenTime is only NULL once

		// Regardless of new bar, update the held bar time
		priorBarOpenTime = currentBarOpenTime;
	}

	return isNewBar;
}

bool IsPendingBuySignal(double &price,double &sl,double &tp)
{
	// STRATEGY
	if ( ! IsDoji(g_rates[1]) ) return false;

	price=g_rates[1].high;
	sl = g_rates[1].low - SL_ATR_FACTOR * g_atr[1];

	return true;
}

bool IsPendingSellSignal(double &price,double &sl,double &tp)
{
	// STRATEGY
	if ( ! IsDoji(g_rates[1]) ) return false;

	price=g_rates[1].low;
	sl = g_rates[1].high + SL_ATR_FACTOR * g_atr[1];

	return true;

}

bool OrderStop(		CTrade& trade,
					const string symbol,
					const ENUM_ORDER_TYPE order_type,
					const double volume,
					const double price,
					const double sl = 0.,
					const double tp = 0.	)
{

	//--- additional checking
	if(! TerminalInfoInteger(TERMINAL_TRADE_ALLOWED))
	{
		PrintFormat("%s(%s): Error [TERMINAL_TRADE_ALLOWED]", __FUNCTION__, symbol );
		return false;
	}

	// enum to string
	if ( order_type == ORDER_TYPE_BUY_STOP )
	{
		if(!trade.BuyStop(volume,price,symbol,sl,tp, ORDER_TIME_GTC, NULL, NULL ))
		{
			PrintFormat("%s(%s) Error: Retcode [%d] : '%s'", __FUNCTION__,symbol, trade.ResultRetcode(), trade.ResultComment() );
			return false;
		}
	}
	else if ( order_type == ORDER_TYPE_SELL_STOP )
	{
		if(!trade.SellStop(volume,price,symbol,sl,tp, ORDER_TIME_GTC, NULL, NULL ))
		{
			PrintFormat("%s(%s) Error: Retcode [%d] : '%s'", __FUNCTION__,symbol, trade.ResultRetcode(), trade.ResultComment() );
			return false;
		}
	}
	else { PrintFormat("%s(): Print! Unknown order type [%s]", __FUNCTION__, EnumToString(order_type) ); }

	return true;
}

bool RefreshData(void)
{
	//--- go trading only for first ticks of new bar
	if(CopyRates(_Symbol,_Period,0,NUM_RATES,g_rates)!=NUM_RATES)
	{
		Print("CopyRates of ",_Symbol," failed, no history");
		return false;
	}

	//--- get current ATR
	if ( g_handleAtr != INVALID_HANDLE )
	{
		if(CopyBuffer(g_handleAtr,0,0,NUM_RATES,g_atr)!=NUM_RATES)
		{
			Print("CopyBuffer from iATR failed, no data");
			return false;
		}
	}

	return true;
}

// Determines if we have an open position or not
bool SelectPosition(const string symbol, const ulong magicNumber)
{
	// Do we have an open position?
	if(!PositionSelect(symbol))
	{
		// No position is open for this symbol
		return false;
	}
	else
	{
		// We found a position for this symbol, but is it one of ours?
		return(PositionGetInteger(POSITION_MAGIC)==magicNumber);
	}

	return false;
}
